% =========================================================================
% Copyright (c) 2008-2014 Audio Communication Group, TU Berlin
% 
% This file is part of WhisPER, a package of interacting MATLAB scripts for 
% designing and controlling experiments in the field of auditory perceptive 
% measurement and evaluation.
% 
% WhisPER is free software: you can redistribute it and/or modify it under 
% the terms of the GNU General Public License as published by the Free 
% Software Foundation, either version 3 of the License, or (at your option) 
% any later version. 
% 
% WhisPER is distributed in the hope that it will be useful, but WITHOUT ANY 
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
% FOR A PARTICULAR PURPOSE.
% See the GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License along 
% with this program. If not, see <http://www.gnu.org/licenses/>. 
% 
% http://www.ak.tu-berlin.de/whisper/ 
% =========================================================================
% 
% This file:
%
% Author :   Simon Ciba, student at FG Audiokommunikation, TU Berlin
% Email  :   sciba2 AT hotmail DOT com
% Date   :    1-Sep-2008
% Updated:   17-Oct-2008 00:00, S.Ciba, structure of parameters
%            14-Nov-2008 16:00, S.Ciba, metadata/comments, bug-fix
%                                       
% =========================================================================
%
% This is an implementation of the adaption and estimation mechanism of a
% transformed staircase procedure following a 3-down/1-up rule, halving step
% size after predefined reversals and estimating the threshold by calcula-
% ting "midrun estimates" for for a predetermined number of last consecutive 
% reversals.
%
%
% bibliography:
%
% Cornsweet, T. N. (1962). The Staircase-Method in Psychophysics. American
% Journal of Psychology, 75, S. 485491.
%
% Levitt, H. (1971). Transformed up-down methods in psychoacoustics. Journal
% of the Acoustical Society of America, 49, Nr. 2, S. 467477.
% 
% -------------------------------------------------------------------------
%
% [x_next, x_est]
%
% = staircase3d1u(x_hist, r_hist, range, nrmax, halvings, numlastr)
%
% 
% -------------------------------------------------------------------------
% 
% Ausgabe:
%   x_next (Integer): Reizstrkenwert fr die Darbietung im nchsten
%                     Trial einer 3-down/1-up Regel folgend. 
%
%   x_est (Integer):  Schtzung der Schwelle durch Mittelwertbildung der 
%                     Reizstrkenwerte bei den numlastr letzten aufeinander-
%                     folgenden Reversals. (x_est = 0: das Verfahren wurde 
%                     noch nicht terminiert, d.h. es liegt noch keine 
%                     Schtzung vor.)
%
% Eingabe: 
%   x_hist (Integer-Array): enthlt die Reizstrkenwerte smtlicher bisher
%                           dargebotener Stimuli
%   r_hist (Integer-Array): enthlt die zugehrigen Antworten 
%                           (+1: positiv, -1: negativ)
%   range (Integer): legt den Wertebereich [1:range] der
%                    Reizstrke fest. Die minimale Schrittweite ist mit 1 
%                    vorgegeben.
%
%   nrmax (Integer): maximale Anzahl an Umkehrungen (Reversals) des Tracks,
%                    bei deren Erreichen das Verfahren terminiert wird.
%   halvings (Integer-Array): enthlt die Indizes der Umkehrpunkte, bei 
%                             denen Halbierungen der Schrittweite erfolgen 
%                             sollen.
%   numlastr (Integer): Anzahl der letzten aufeinanderfolgenden Reversals, 
%                       welche in die Schtzung des Endergebnisses eingehen 
%                       sollen.  
%
% -------------------------------------------------------------------------
%
% Hinweis: Fr die Funktionalitt des vorliegenden Algorithmus darf numlastr
% nrmax nicht bersteigen. Aus methodischen Grnden sollte numlastr auerdem 
% eine gerade (positive) Zahl sein (vgl. "mid-run estimates" Levitt, 1971).
% Es sollten ferner nur solche Reversals in die Mittelwertbildung einbezogen 
% werden, welche nach Erreichen der minimalen Schrittweite von 1 auftreten. 
%
% -------------------------------------------------------------------------



function [x_next, x_est] = staircase3d1u(x_hist, r_hist, range, nrmax, halvings, numlastr)


% Parameter
Ntrials = length(x_hist); % Anzahl bisher durchgefhrter Trials
numhalv_total = length(halvings); % Anzahl der insgesamt vorgesehenen Halbierungen
dx_ini = 2^numhalv_total; % Anfangswert der Schrittweite

% Initialisierungen
latest_change = 0; % Indikator fr zuletzt erfolgte Reizstrkennderung 
                   % (0: noch keine nderung im bisherigen Verlauf,
                   % 1: Erhhung, -1: Verminderung)
nr = 0; % Zhler fr Reversals des Tracks
numhalv = 0; % Zhler fr aktuell zu erfolgende Halbierungen
termflag = 0; % Indikator fr Terminierung des Verfahrens mit dem aktuellen 
              % Trial, d.h. fr das Erreichen der maximalen Anzahl nrmax an
              % Reversals. (1: beenden, 0: nicht beenden)
x_r = zeros(nrmax, 1); % Speicherung der Reizstrkenwerte bei smtlichen Reversals
T = 1; % Zhler fr Trials bei gleichbleibender Reizstrke
x_est = 0;



%                              * * * * *
%                    Analyse des bisherigen Tracks

for trial = 2:Ntrials

    % Reizstrkenwechsel?
    if x_hist(trial) ~= x_hist(trial-1)
        T=1;
        % Art des Reizstrkenwechsels
        if x_hist(trial) > x_hist(trial-1)
            change = 1; % Erhhung
        else
            change = -1; % Verminderung
        end
        
    else
        change = 0; % bleibt gleich
        T=T+1;
    end
    
    % Reversal?
    if (change ~= 0) && (latest_change ~=0) && (latest_change ~= change)
       nr = nr + 1;
       x_r(nr) = x_hist(trial-1);
       latest_change = change;
       if isempty(find(halvings==nr))==0
           numhalv = numhalv + 1;
       end
    elseif (change ~= 0) && (latest_change == 0)
        latest_change = change;
    end

end

%                              * * * * * 
%                          Nchster Schritt

% Adaptionsregel: Entscheidung ber die Richtung des nchsten Schrittes 
% (3-Down/1-UP)
if (Ntrials>=3) 
    if (T>=3) && (r_hist(Ntrials) == 1) && (r_hist(Ntrials-1) == 1) && (r_hist(Ntrials-2) == 1)
        this_change = -1;
    elseif r_hist(Ntrials) == -1
        this_change = +1;
    else
    this_change = 0;
    end
else
    this_change = 0;
end

% Reversal?
if (this_change ~= 0) && (this_change ~= latest_change) && (latest_change ~=0) 
    nr = nr + 1;
    x_r(nr)=x_hist(Ntrials);
    % ggf. Halbierung der Schrittweite
    if isempty(find(halvings==nr))==0
           numhalv = numhalv + 1;
    end
end

% Terminierung?
if nr >= nrmax
    termflag = 1;
end

% Berechnung der Gre des nchsten Schrittes 
% x_hist(Ntrials) --> x_hist(Ntrials + 1)           
dx = round(dx_ini/(2^numhalv));

% Berechnung des nchsten Reizstrkenwertes
x_next = x_hist(Ntrials) + this_change * dx;

% Verhalten an den Rndern
if x_next > range
    x_next = range;
end
if x_next < 1
    x_next = 1;
end

%                              * * * * *
%                 Berechnung der Schwellenschtzung

if termflag == 1
    x_est = mean(x_r((nrmax-numlastr+1):nrmax));
end



